package com.nisum.javaSparkExample.kafka;

import com.nisum.javaSparkExample.model.Customers;
import org.apache.spark.sql.*;
import org.apache.spark.sql.types.StructType;

import static org.apache.spark.sql.functions.col;
import static org.apache.spark.sql.functions.expr;

public class ReadKafkaToCsv {
    public static void main(String[] args) {


        SparkSession sparkSession = SparkSession.builder()
                .master("local[*]")
                .appName("kafkaToCsvProcess")
                .config("spark.ui.enabled", true)
                .config("spark.sql.shuffle.partitions", 2)
                .getOrCreate();

        Dataset<Customers> customermongoData = sparkSession.read()
                .format("mongo")
                .option("uri", "mongodb://localhost:27017/mydb.customers")
                .load().as(Encoders.bean(Customers.class));
        System.out.println(" Customers Data from MongoDB:");
        customermongoData.select("customerId", "name", "gender", "transaction_amount").show(false);


        Dataset<Row> kafkaDataFrame =sparkSession.read()
                .format("kafka")
                .option("kafka.bootstrap.servers", "localhost:9092")
                .option("subscribe", "customer-topic")
                .option("group.id", "customer-group")
                .option("auto.offset.reset", "latest")
                .load();
        Dataset<Row> kafkaData = kafkaDataFrame.selectExpr("CAST(value as String) as message");
        StructType customer = Encoders.bean(Customers.class).schema();
        Dataset<Customers> dataset = kafkaData.select(functions.from_json(functions.col("message"), customer).as("data"))
                .select(
                        "data.customerId","data.name","data.gender","data.transaction_amount"
                )
                .as(Encoders.bean(Customers.class));
        dataset.show(false);
//        Dataset<Customers> kafkaDataFrame1 =dataset.as(Encoders.bean(Customers.class));
//
//
////        Dataset<Row> matchingRecords = customermongoData.join(kafkaDataFrame,customermongoData.col("customerId")
////				.equalTo(kafkaDataFrame.col("customerId")), "inner");
//             kafkaDataFrame1.show(false);

        Dataset<Row> joinedData = dataset.as("k")
                .join(customermongoData.as("m"), expr("k.customerId = m.customerId"), "left_outer")
                .select(
                        col("m.customerId"),
                        col("k.name").alias("name"),
                        col("m.gender"),
                        col("k.transaction_amount").alias("transaction_amount")
                );
        // Check if any records exist after joining
        if (joinedData.count() > 0) {
            // Records with matching "customerId" found, perform update operation

            // For example, if you want to update the "name" and "transaction_amount" in customermongoData with values from kafkaDataFrame:
            customermongoData = customermongoData.as("m")
                    .join(dataset.as("k"), col("m.customerId").equalTo(col("k.customerId")), "left_outer")
                    .select(
                            col("m.customerId"),
                            col("k.name").alias("name"),
                            col("m.gender"),
                            col("k.transaction_amount").alias("transaction_amount")
                    ).as(Encoders.bean(Customers.class));
                    //.drop("customerId").as(Encoders.bean(Customers.class)); // Drop the duplicate customerId column from kafkaDataFrame

        } else {
            // Records with matching "customerId" not found, save to CSV file
            customermongoData.write().csv("D:/kafkaspark/customers.csv");
        }
        // Show the updated customermongoData
        System.out.println("Customers Data after update:");
        customermongoData.show(false);


    }
}
